@@ -683,32 +683,63 @@ class Context():
         # cascade tables bundles all cascade data
         if self.config_cascade_table_output == 'SEPARATE':
             # no bundled cascade table
+            self.file_cascade_report_detail_all = None
             self.csv_cascade_report_detail_all = None
+            self.file_cascade_report_detail_primary_all = None
             self.csv_cascade_report_detail_primary_all = None
+            self.file_cascade_report_lattice_sites_all = None
             self.csv_cascade_report_lattice_sites_all = None
+            self.file_cascade_orgiv_all = None
             self.csv_cascade_orgiv_all = None
+            self.file_cascade_ranger_all = None
             self.csv_cascade_ranger_all = None
         else: # 'BUNDLE' or 'BOTH'
-            self.csv_cascade_report_detail_all = csv.writer(
-                (self.outputdir / 'description_of_cascade_all.csv').open('w', newline=''))
+            self.file_cascade_report_detail_all\
+                    = (self.outputdir / 'description_of_cascade_all.csv').open('w', newline='')
+            self.csv_cascade_report_detail_all = csv.writer(self.file_cascade_report_detail_all)
             self.csv_cascade_report_detail_all.writerow(csv_header.cascade_report_detail_all)
 
-            self.csv_cascade_report_detail_primary_all = csv.writer(
-                (self.outputdir / 'description_of_cascade_primary_recoil_all.csv').open('w', newline=''))
+            self.file_cascade_report_detail_primary_all\
+                    = (self.outputdir / 'description_of_cascade_primary_recoil_all.csv').open('w', newline='')
+            self.csv_cascade_report_detail_primary_all = csv.writer(self.file_cascade_report_detail_primary_all)
             self.csv_cascade_report_detail_primary_all.writerow(csv_header.cascade_report_detail_all)
 
-            self.csv_cascade_report_lattice_sites_all = csv.writer(
-                (self.outputdir / 'lattice_sites_all.csv').open('w', newline=''))
+
+            self.file_cascade_report_lattice_sites_all\
+                    = (self.outputdir / 'lattice_sites_all.csv').open('w', newline='')
+            self.csv_cascade_report_lattice_sites_all = csv.writer(self.file_cascade_report_lattice_sites_all)
             self.csv_cascade_report_lattice_sites_all.writerow(csv_header.cascade_report_lattice_sites_all)
 
-            self.csv_cascade_orgiv_all = csv.writer(
-                (self.outputdir / 'distant_iv_pairs_all.csv').open('w', newline=''))
+            self.file_cascade_orgiv_all = (self.outputdir / 'distant_iv_pairs_all.csv').open('w', newline='')
+            self.csv_cascade_orgiv_all = csv.writer(self.file_cascade_orgiv_all)
             self.csv_cascade_orgiv_all.writerow(csv_header.cascade_orgiv_all)
 
-            self.csv_cascade_ranger_all = csv.writer(
-                (self.outputdir / 'primary_recoil_ranges_all.csv').open('w', newline=''))
+            self.file_cascade_ranger_all = (self.outputdir / 'primary_recoil_ranges_all.csv').open('w', newline='')
+            self.csv_cascade_ranger_all = csv.writer(self.file_cascade_ranger_all)
             self.csv_cascade_ranger_all.writerow(csv_header.cascade_ranger_all)
 
+    def close_bundled_files_explicitly(self):
+        '''explicitly close csv files, which are created on BUNDLE and BOTH mode'''
+        if self.file_cascade_report_detail_all:
+            self.file_cascade_report_detail_all.close()
+        self.csv_cascade_report_detail_all = None
+
+        if self.file_cascade_report_detail_primary_all:
+            self.file_cascade_report_detail_primary_all.close()
+        self.csv_cascade_report_detail_primary_all = None
+
+        if self.file_cascade_report_lattice_sites_all:
+            self.file_cascade_report_lattice_sites_all.close()
+        self.csv_cascade_report_lattice_sites_all = None
+
+        if self.file_cascade_orgiv_all:
+            self.file_cascade_orgiv_all.close()
+        self.csv_cascade_orgiv_all = None
+
+        if self.file_cascade_ranger_all:
+            self.file_cascade_ranger_all.close()
+        self.csv_cascade_ranger_all = None
+
     # contexts durinng parsing cascade 
     def init_cascade_contexts(self, cascade, group, number):
         '''update cascade_index and cascadedir, then create cascadedir'''
