@@ -23,13 +23,21 @@ EOF: "$EOF"
 %import common.INT
 %import common.SIGNED_INT
 %import common.SIGNED_FLOAT
-_S: /\s+/
+_SS: " " // single whitespace
+_S: _SS+ // sequence of whitespace
 int: _S? SIGNED_INT      -> int // int with left padding
 pint: SIGNED_INT -> int // int without padding
 float: _S? SIGNED_FLOAT _S? -> float // float with left and right padding
 pfloat: SIGNED_FLOAT -> float // float without padding
 '''
 
+fixedwidth_int_values = [3, 4, 5]
+def gen_fixedwidth_int_grammar(width):
+    return f'''FWINT{width:d}: /([ \\d-]{{{width-1:d}}}\\d)|(\\*{{{width:d}}})/
+    fwint{width:d}: FWINT{width:d} -> fwint{width:d}'''
+
+grammar_fixedwidth_int = '\n'.join([gen_fixedwidth_int_grammar(i) for i in fixedwidth_int_values])
+
 grammar_ttls_momx = '''\
 // general/titles.mfs TTLS0120
 TTLS_MOMX_1: "Mean"
@@ -376,9 +384,20 @@ ELEM: "H" | "He"
     | "Mt" | "Ds" | "Rg" | "Cn" | "Uut" | "Fl" | "Uup" | "Lv" | "Uus" | "Uuo"
 '''
 
-class Transformaer(lark.Transformer):
+class Transformer(lark.Transformer):
     def int(self, args):
         return int(args[0])
 
     def float(self, args):
         return float(args[0])
+
+# add fwint{width} recievers
+def gen_fixedwidth_int_reciever(width):
+    def func(self, args):
+        if args[0] == '*'*width:
+            args[0] = args[0].replace('*', '9')
+        return int(args[0])
+    return func
+
+for i in fixedwidth_int_values:
+    setattr(Transformer, f'fwint{i:d}', gen_fixedwidth_int_reciever(i))
